
/**
 * @(#)Game.java	1.7 99/04/20
 *
 * Description: Game class keeps information about current game
 *              provides game control methods.
 *
 * Copyright (c) 1996-1999 James Begole.  All Rights Reserved.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY 
 * OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole SHALL NOT BE LIABLE
 * FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES"). James Begole
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 *
 * @author   James "Bo" Begole
 * @author   Rich Burridge
 * @version  1.5 - 20th April 1999. 
 */

package examples.ppong;

import com.sun.media.jsdt.*;
import java.io.*;
import java.net.*;

class Game implements PpongDebugFlags {

    private Ppong         owner          = null;
    private PpongCourt    court          = null;
    private GeneralClient client         = null;
    private String        serverHost     = null;
    private int           port           = 0;
    private String        type           = null;
    private boolean       inProgress     = false;
    public  boolean       gameInProgress = false;
    public  long          number         = 0;

    private Session session     = null;
    private Channel channel     = null;
    private String  clientName  = null;
    private Client  scoreClient = null;
    public  String  myHost      = null;


    public 
    Game(Ppong ppong, PpongCourt court, 
	 String serverHost, int port, String type) {
        owner           = ppong;
        this.serverHost = serverHost;
        this.port       = port;
        this.type       = type;
	this.court      = court;

        client = new GeneralClient(court);
        client.start(serverHost, port, type);

        court.start();
        court.setMessage("Connecting to Server at: " + serverHost);

// Set up the statistics output.

        try {
            URLString url = URLString.createSessionURL(serverHost, port,
			                               type, "PpongSession");

	    try {
		InetAddress localhost = InetAddress.getLocalHost();
                int n = rrange(20, 100) * rrange(1, 10000);

                myHost = localhost.getHostName();
                clientName = "ScoreClient@" + myHost + n;
            } catch (UnknownHostException uhe) {
                owner.showStatus("Can not determine local host");
            }

	    scoreClient = (Client) new PpongClient(clientName);
	    session = SessionFactory.createSession(scoreClient, url, true);
            channel = session.createChannel(scoreClient, "ScoreChannel",
                                            true, true, true);
        } catch (JSDTException e) {        
            System.out.println("Game: constructor: Could not connect to '" + 
			       owner.remoteHost + "'.");
            e.printStackTrace();
	    return;
        }
    }


/* Generate a random number between low and high. */

    private int
    rrange(int low, int high) {
        return((int)(Math.random() * (high - low + 1) + low));
    }


    public void 
    start() {
        if (!inProgress) {
            court.init();
            owner.startButton.toggle();
            owner.playSound(Ppong.S_MUSIC);
            inProgress = true;
        }
    }


    public void 
    stop() {
        pause();
        court.stop();
    }


    public void 
    end() {
        if (Game_Debug) {
            System.out.println("Ending Game 1: GameInProgress =" +
			       gameInProgress);
            System.out.println("inProgress = " + inProgress);
	}

        if (inProgress) {
            if (Game_Debug) {
                System.out.println("Ending Game");
	    }

            client.stop();
            owner.startButton.toggle();
            owner.stopSound(Ppong.S_MUSIC);
            inProgress = false;

            try {
		if (gameInProgress) {    // Report Score to Server.
                    gameInProgress = false;
                    send(Message.writeScore(number, court.client.getName(), 
			 		    court.myScore, court.opponentHost,
                                            court.opponentScore));
		}
                channel.leave(scoreClient);
                session.leave(scoreClient);
            } catch (JSDTException e) {
                System.out.println("Game: end: exception: " + e);
            }

            try {
                session.close(true);
	    } catch (JSDTException e) {
                System.out.println("Game: end: exception: " + e);
            }

	    if (owner.pauseButton.getLabel().equals("Resume")) {
                owner.pauseButton.toggle();
	    }

            owner.pauseButton.setEnabled(false);
            owner.soundsButton.setEnabled(false);
        }
    }


    public void
    send(byte[] message) {
        try {
            channel.sendToClient(scoreClient, "PpongServer", new Data(message));
        } catch (JSDTException e) {
            System.out.println("Game: send: exception: " + e);
        }   
    }


    public void 
    pause() {
        if (gameInProgress) {
            court.stopBallInPlay();
	}
    }


    public void 
    resume() {
        if (gameInProgress) {
            court.putBallInPlay();
	}
    }
}
