
/**
 * @(#)GeneralClient.java	1.6 99/04/20
 *
 * Copyright (c) 1996-1999 James Begole.  All Rights Reserved.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY 
 * OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole SHALL NOT BE LIABLE
 * FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES"). James Begole
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 *
 * @author   James "Bo" Begole
 * @author   Rich Burridge
 * @version  1.5 - 20th April 1999. 
 */

package examples.ppong;

import com.sun.media.jsdt.*;
import java.io.*;
import java.net.*;

class 
GeneralClient implements Runnable, PpongDebugFlags {

    private ConnectionHandler owner;
    private String            host;
    private int               port;
    private String            type;
    private Session           session      = null;
    private Channel           channel      = null;
    private String            clientName   = null;
    private Client            gameClient   = null;
    private String            myHost       = null;

    private Thread            life         = null;
    private boolean           runState     = false;
    private boolean           connectState = false;


    public 
    GeneralClient(ConnectionHandler theOwner) {
        owner = theOwner;
    }


    public void 
    start(String host, int port, String type) {
        owner.debugLog("GeneralClient:start", this);

        runState  = true;
        this.host = host;
        this.port = port;
	this.type = type;

        if (life == null) {
            life = new Thread(this);
        }
        life.start();
    }


    public void 
    stop() {
        owner.debugLog("GeneralClient:stop", this);

        if (life != null) {
            life.stop();
	}
        shutdown();
    }


    public boolean 
    isRunning() {
        owner.debugLog("GeneralClient:isRunning", this);

        return(runState);
    }


    public boolean 
    isConnected() {
        owner.debugLog("GeneralClient:isConnected", this);

        return(connectState);
    }


    public void 
    run() {
        owner.debugLog("GeneralClient:run", this);

        owner.changeStatus("Connecting to '" + host + "' on port '" + 
                           port + "'...", this);

        try {
            URLString url = URLString.createSessionURL(host, port,
                                                       type, "PpongSession");

            try {
                InetAddress localhost = InetAddress.getLocalHost();
                int n = rrange(20, 100) * rrange(1, 10000);

                myHost = localhost.getHostName();
                clientName = "GameClient@" + myHost + n;
            } catch (UnknownHostException uhe) {
                System.out.println("GeneralClient: run: exception: " + uhe);
            }

            gameClient = (Client) new PpongClient(clientName);
            session    = SessionFactory.createSession(gameClient, url, true);
            channel    = session.createChannel(gameClient, "GameChannel",
                                               true, true, true);
            channel.addConsumer(gameClient, (ChannelConsumer) owner);
        } catch (JSDTException e) {        
            owner.connectionMishap("GeneralClient: run: Could not connect to '" + 
				   host + "'.", this);
            e.printStackTrace();
	    shutdown();
            return;
        }       

        owner.connectionUp("Connected.", this);

        connectState = true;
        owner.handleSession(channel, gameClient, this);
        connectState = false;
        owner.connectionDown("Disconnected.", this);
    }


/* Generate a random number between low and high. */

    private int
    rrange(int low, int high) {
        return((int)(Math.random() * (high - low + 1) + low));
    }


    public void 
    shutdown() {
        owner.debugLog("GeneralClient:shutdown", this);

        life     = null;
        runState = false;
        if (connectState) {
            try {
		channel.leave(gameClient);
	        session.leave(gameClient);
	    } catch (JSDTException e) {
                owner.connectionMishap("Could not shutdown '" + "'.", this);
            }

	    try {
		session.close(true);
            } catch (JSDTException e) {
                owner.connectionMishap("Could not shutdown '" + "'.", this);
            }

            connectState = false;
        }
    }
}
