
/*
 * @(#)StockRecord.java	1.17 99/01/07
 *
 * Copyright (c) 1993-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package examples.stock;

import com.sun.media.jsdt.*;
import java.io.*;
import java.awt.*;

/**
 * Stock quote records on the server.
 *
 * @version     1.5 - 7th January 1999
 * @author      Amith Yamasani
 * @author      Rich Burridge
 */

public class
StockRecord implements StockDebugFlags {

    /** The server-side client associated with this stock symbol. */
    private Client client;

    /** The Shared byte array associated with this stock symbol. */
    private ByteArray sba;

    private boolean isValid;

    private String symbol, time, value, change, quotes;


    public
    StockRecord(Client client, ByteArray sba) {
        if (StockRecord_Debug) {
            System.err.println("StockRecord: constructor:" +
				" client: "            + client +
				" shared byte array: " + sba);
        }

	this.client = client;
	this.sba    = sba;

	symbol = sba.getName();
    }


    public ByteArray
    getByteArray() {
        if (StockRecord_Debug) {
            System.err.println("StockRecord: getByteArray.");
        }

	return(sba);
    }


    public void
    setChange(String change) {
        if (StockRecord_Debug) {
            System.err.println("StockRecord: setChange:" +
                                " change: " + change);
        }

        this.change = change;
    }


    public void
    setTime(String time)  {
        if (StockRecord_Debug) {
            System.err.println("StockRecord: setTime:" +
                                " time: " + time);
        }

        this.time = time;
    }


    public void
    setValid(boolean isValid) {
        if (StockRecord_Debug) {
            System.err.println("StockRecord: setValid:" +
                                " valid?: " + isValid);
        }

        this.isValid = isValid;
    }


    public void
    setValue(String value)  {
        if (StockRecord_Debug) {
            System.err.println("StockRecord: setValue:" +
                                " value: " + value);
        }

	this.value = value;
    }


    public void
    clearQuotes() {
        if (StockRecord_Debug) {
            System.err.println("StockRecord: clearQuotes:" +
                                " value: " + value);
        }

        quotes = new String("");
    }


    public void
    addQuote(String newQuote) {
        if (StockRecord_Debug) {
            System.err.println("StockRecord: addQuote:" +
                                " new quote: " + newQuote);
        }

        quotes = quotes.concat(newQuote).concat(" ");
    }


    public void
    update() {
        ByteArrayOutputStream baos;
	DataOutputStream dos;

        if (StockRecord_Debug) {
            System.err.println("StockRecord: update.");
	}

	baos = new ByteArrayOutputStream();
	dos  = new DataOutputStream(baos);

        try {
	    dos.writeUTF(symbol);
            dos.writeBoolean(isValid);
	    if (isValid) {
		dos.writeUTF(time);
                dos.writeUTF(value);
                dos.writeUTF(change);
                dos.writeUTF(quotes);
	    }
	    dos.flush();
	} catch (IOException e) {
            System.err.println("StockRecord: update: exception: " + e);
            if (StockRecord_Debug) {
                e.printStackTrace();
            }
	}

	try {
	    sba.setValue(client, baos.toByteArray());
	} catch (JSDTException je) {
            System.err.println("StockRecord: update: exception: " + je);
            if (StockRecord_Debug) {
                je.printStackTrace();
            }
        }
    }
}
