
/*
 * @(#)AudioConsumer.java	1.13 99/01/07
 *
 * Copyright (c) 1993-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package examples.sound;

import com.sun.media.jsdt.*;
import java.util.Vector;

/**
 * This is just a simple server/client based proof-of-concept
 * implementation to test UDP channels.
 *
 * The server continuously sends a sound file over a UDP channel
 * to each client joined to the channel. These in turn, play the sound on
 * the workstation speaker.
 *
 * Code to implement this is based upon the audio jukebox work of Befhar
 * Razavi.
 *
 * @version     1.5 - 7th January 1999
 * @author      Rich Burridge
 */

public class
AudioConsumer implements ChannelConsumer, SoundDebugFlags {

    /** The name of this channel consumer. */
    protected String name;

    /** The buffered audio data. */
    Vector audioBuffer;

    private int jitterBuffer = 2;


    public
    AudioConsumer(String name) {
        if (AudioConsumer_Debug) {
            System.err.println("AudioConsumer: constructor:" +
			       " name: " + name);
        }

        this.name   = name;
	audioBuffer = new Vector(200, 50);
    }
 
 
    public synchronized void
    dataReceived(Data data) {
        int        length   = data.getLength();
	byte[]     theBytes = new byte[length];
	AudioClick click;

        if (AudioConsumer_Debug) {
            System.err.println("AudioConsumer: dataReceived: length:" + length);
        }

        length   = data.getLength();
        theBytes = new byte[length];
        System.arraycopy(data.getDataAsBytes(), 0, theBytes, 0, length);
        click = new AudioClick(theBytes, 0, length);
        audioBuffer.addElement(click);
    }


/** Return the audio click at the head of the queue. If the queue is empty,
 *  wait a little bit to allow the buffer to fill up some before resuming.
 */

    public AudioClick
    get() {
	AudioClick click = null;

        if (AudioConsumer_Debug) {
            System.err.println("AudioConsumer: get.");
        }

	if (audioBuffer.size() == 0) {
	    try {
		Thread.sleep(50 * jitterBuffer);
	    } catch (Exception e) {
                System.err.println("AudioConsumer: get: exception " + e);
                if (AudioConsumer_Debug) {
                    e.printStackTrace();
                }
	    }
	}

	if (audioBuffer.size() == 0) {
	    return(null);
	} 

	try {
	    click = (AudioClick) audioBuffer.firstElement();
	    audioBuffer.removeElementAt(0);
	} catch (Exception e) {
	    System.err.println("AudioConsumer: get: exception " + e);  
	    if (AudioConsumer_Debug) {
		e.printStackTrace();
	    }
	    return(null);
	}
        
	return(click);
    }
}
