
/*
 * @(#)SendSoundInfo.java	1.13 99/01/07
 *
 * Copyright (c) 1993-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package examples.sound;

import com.sun.media.jsdt.*;
import java.io.*;

/**
 * This is just a simple server/client based proof-of-concept
 * implementation to test UDP channels.
 *
 * The server continuously sends a sound file over a UDP channel
 * to each client joined to the channel. These in turn, play the sound on
 * the workstation speaker.
 *
 * @version     1.5 - 7th January 1999
 * @author      Rich Burridge
 */

public class
SendSoundInfo extends Thread implements SoundDebugFlags {

    /** The channel to send the audio file information over. */
    private Channel channel;

    /** The client sending the audio file info. */
    private Client client;

    /** The name of the client to receive the audio file info. */
    private String receivingClientName;

    /* The audio information. */
    private String audioFileName;
    private int    sampleRate;
    private int    encoding;
    private int    channels;


    public
    SendSoundInfo(Channel channel, Client client, String receivingClientName,
		  String audioFileName, int sampleRate, 
		  int encoding, int channels) {
        if (SendSoundInfo_Debug) {
            System.err.println("SendSoundInfo: constructor: " +
				" channel: "          + channel +
				" client: "           + client +
				" receiving client: " + receivingClientName +
				" audio file name: "  + audioFileName +
				" sample rate: "      + sampleRate +
				" encoding: "         + encoding +
				" channels: "         + channels);
        }
 
        this.channel             = channel;
        this.client              = client;
	this.receivingClientName = receivingClientName;
	this.audioFileName       = audioFileName;
	this.sampleRate          = sampleRate;
        this.encoding            = encoding;
	this.channels            = channels;
    }


    public void
    run() {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream      dos  = new DataOutputStream(baos);

        if (SendSoundInfo_Debug) {
            System.err.println("SendSoundInfo: run.");
	}

        try {
	    dos.writeUTF(audioFileName);
            dos.writeInt(sampleRate);
            dos.writeInt(encoding);
            dos.writeInt(channels);
            dos.flush();

	    channel.sendToClient(client, receivingClientName,
				 new Data(baos.toByteArray()));
        } catch (Exception e) {
            System.err.println("SendSoundInfo: run: exception " + e);
            if (SendSoundInfo_Debug) {
                e.printStackTrace();
            }
        }
    }
}
