
/*
 * @(#)SoundListener.java	1.17 99/01/07
 *
 * Copyright (c) 1993-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package examples.sound;

import com.sun.media.jsdt.*;
import com.sun.media.jsdt.event.*;

/**
 * This is just a simple server/client based proof-of-concept
 * implementation to test UDP channels.
 *
 * The server continuously sends a sound file over a UDP channel
 * to each client joined to the channel. These in turn, play the sound on
 * the workstation speaker.
 *
 * @version     1.5 - 7th January 1999
 * @author      Rich Burridge
 */

public class
SoundListener extends ChannelAdaptor implements SoundDebugFlags {

    /** The name of the audio file to continuously play on this channel. */
    private String audioFileName = null;

    /** The server-side client already joined to this channel. */
    private Client client;

    /** Channel to send the audio file over. */
    private Channel audioChannel;

    /** The command channel to send sound file information over. */
    private Channel commandChannel;

    /** The thread playing the audio file. */
    private PlayAudioFile paf = null;

    /** The audio file reader (for sample rate, encoding and channels). */
    private AudioFileReader afr;

    /** The number of clients currently joined to this channel. */
    private int noClients = 0;


    public
    SoundListener(String audioFileName, Client client, 
		  Channel audioChannel, Channel commandChannel) {
        if (SoundListener_Debug) {
            System.err.println("SoundListener: constructor: " +
				" audio file name: " + audioFileName +
				" client: "          + client +
				" audio channel: "   + audioChannel +
				" command channel: " + commandChannel);
        }

        this.audioFileName  = audioFileName;
	this.client         = client;
	this.audioChannel   = audioChannel;
	this.commandChannel = commandChannel;
    }


    public void
    channelJoined(ChannelEvent event) {
	Channel audioChannel = event.getChannel();
        SendSoundInfo ssfi = null;

        if (SoundListener_Debug) {
            System.err.println("SoundListener: channelJoined:" +
                               " event: " + event);
        }

	noClients++;

/* If this is the first client joining the channel, fire off the thread
 * to play the audio file.
 */
	if (noClients == 1) {
	    if (paf == null) {
		paf = new PlayAudioFile(audioChannel,
					client, audioFileName);
		afr = paf.getReader();
                System.err.println("Starting to play the audio file.");
		paf.start();
	    }
	}

/* Send the sound file information to the newly joined client. */

	ssfi = new SendSoundInfo(commandChannel, client,
				 event.getClientName(),
				 audioFileName,
				 afr.getSampleRate(), afr.getEncoding(),
				 afr.getChannels());
	ssfi.start();

        if (noClients == 1) {
	    System.err.println("Resuming the audio file.");
            paf.resume();
	}
    }    


    public void
    channelLeft(ChannelEvent event) {
        if (SoundListener_Debug) {
            System.err.println("SoundListener: channelLeft:" +
                               " event: " + event);
        }

        noClients--;
 
/* If there are no clients currently joined to this channel, then turn off
 * the audio file.
 */
        if (noClients == 0) {
            System.err.println("Suspending the audio file.");
 
            paf.suspend();
        }
    }
}
