
/*
 * @(#)DrawingArea.java	1.16 99/01/07
 *
 * Copyright (c) 1993-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package examples.whiteboard;

import com.sun.media.jsdt.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.applet.Applet;

/**
 * This is just a simple server/client based proof-of-concept
 * implementation to make sure that the Shared Data classes have
 * all the required API in them.
 *
 * It's based on WhiteBoard; a test applet for the game server from
 * daniel@vpro.nl. It's been rewritten to use the Java Shared Data API,
 * and to make it easier to modify and maintain.
 *
 * @version     1.5 - 7th January 1999
 * @author      daniel@vpro.nl
 * @author      Rich Burridge
 */

public class
DrawingArea extends Panel implements KeyListener, MouseListener, 
				MouseMotionListener, WhiteBoardDebugFlags {

    /** The WhiteBoardUser for this menu canvas. */
    WhiteBoardUser wbu;

    /** Offscreen backing store image of the menu controls. */
    private Image image = null;

    /** The Graphics context for the offscreen image. */
    Graphics gc;

    /** The width of the white board drawing area canvas. */
    private int width;

    /** The height of the white board drawing area canvas. */
    private int height;


    public
    DrawingArea(int width, int height, WhiteBoardUser client) {
        if (DrawingArea_Debug) {
            System.err.println("DrawingArea: constructor:" +
                                " width: "  + width +
                                " height: " + height +
                                " client: " + client);
        }
 
        this.width  = width;
        this.height = height;
        wbu         = client;
	addKeyListener(this);
	addMouseListener(this);
	addMouseMotionListener(this);
    }
 
 
    public Dimension
    getPreferredSize() {
        return(new Dimension(width, height));
    }


    public void mouseReleased(MouseEvent event) {}

    public void mouseEntered(MouseEvent event) {}

    public void mouseExited(MouseEvent event) {}

    public void mouseClicked(MouseEvent event) {}

    public void mouseMoved(MouseEvent event) {}

    public void 
    mousePressed(MouseEvent event) {
        int x = event.getX();
	int y = event.getY();

	if (DrawingArea_Debug) {
            System.err.println("DrawingArea: mousePressed.");
        }

        if (wbu.drawtype == 0) {                        /* DOT. */
            wbu.writeLine("DOT " + x + " " + y + "\n");
        } else if (wbu.drawtype == 1) {                 /* LINE. */
            if (wbu.pointx1 == 0) {
                wbu.pointx1 = x;
                wbu.pointy1 = y;
            } else {
                wbu.writeLine("LINE " + wbu.pointx1 + " " +
                              wbu.pointy1 + " " + x + " " + y + "\n");
                wbu.pointx1 = 0;
            }
        } else if (wbu.drawtype == 2) {                 /* TEXT. */
            if (wbu.pointx1 == 0) {
                wbu.pointx1   = x;
                wbu.pointy1   = y;
                wbu.textmode  = true;
                requestFocus();
            }
        } else if (wbu.drawtype == 3) {                 /* CIRCLE. */
            if (wbu.pointx1 == 0) {
                wbu.pointx1 = x;
                wbu.pointy1 = y;
            } else {
                wbu.writeLine("CIRCLE " + wbu.pointx1 + " " +
                                wbu.pointy1 + " " + x + " " + y + "\n");
                wbu.pointx1 = 0;
            }
        }

        wbu.controls.repaint();
        repaint();
    }


    public void 
    mouseDragged(MouseEvent event) {
        if (DrawingArea_Debug) {
            System.err.println("DrawingArea: mouseDragged.");
        }

        if (wbu.drawtype == 0) {
            wbu.writeLine("DOT " + event.getX() + " " + event.getY() + "\n");
        }

        wbu.controls.repaint();
        repaint();
    }


    public void keyPressed(KeyEvent event) {}


    public void keyTyped(KeyEvent event) {}


    public void 
    keyReleased(KeyEvent event) {
        char keyChar = event.getKeyChar();
	int  keyCode = event.getKeyCode();

	if (DrawingArea_Debug) {
            System.err.println("DrawingArea: keyReleased: " + event);
        }

        if (keyCode != KeyEvent.VK_SHIFT) {
	    wbu.handleKey((int) keyChar);
        }
	wbu.controls.repaint();
        repaint();
    }


    public synchronized void
    update(Graphics g) {
        Dimension d = getSize();

        if (DrawingArea_Debug) {
            System.err.println("DrawingArea: update.");
        }

        if (image == null ||
            image.getWidth(null) != width ||
            image.getHeight(null) != height) {
            image = createImage(width, height);
            if (image != null) {
		if (gc != null) {
		    gc.dispose();
		}
		gc = image.getGraphics();
                gc.setFont(new Font("Helvetica", Font.PLAIN, 24));
                gc.setColor(Color.white);
                gc.fillRect(0, 0, width, height);
	    }
        }
        redraw();
	if (image != null) {
            g.drawImage(image, 0, 0, this);
	}
    }


    public void
    redraw() {
        String drawcommand, tmp;
        int    who, i, j, px1, py1, px2, py2, radius;
        float  k, l, m;
        StringTokenizer tok;
        Image  selectedbrush;

        if (DrawingArea_Debug) {
            System.err.println("DrawingArea: redraw: commandLine is " +
                                wbu.commandLine);
        }

/* Loop for commands, it stays there until no new commands are in the buffer */

        while (wbu.commandLine != null) {
            tok = new StringTokenizer(wbu.commandLine, " \n\r\t");
            wbu.commandLine = null;
            who = Integer.parseInt(tok.nextToken());
            drawcommand = tok.nextToken();

            if (drawcommand.equals("BRUSH")) {

/* Brush change found set new drawimage for that whiteboard. */

                wbu.brushes.put(new Integer(who),
				wbu.bs[Integer.parseInt(tok.nextToken())]);
            } else if (drawcommand.equals("DOT")) {

/* Dot found draw dot in correct brush/color on screen. */

                selectedbrush = (Image) wbu.brushes.get(new Integer(who));

                if (selectedbrush == null) {
		   wbu.brushes.put(new Integer(who), wbu.bs[0]);
		}
                selectedbrush = (Image) wbu.brushes.get(new Integer(who));

		gc.drawImage(selectedbrush,
                    Integer.parseInt(tok.nextToken()) -
                                     (selectedbrush.getWidth(this) / 2),
                    Integer.parseInt(tok.nextToken()) -
                                     (selectedbrush.getHeight(this) / 2),
                    this);
            } else if (drawcommand.equals("TEXT")) {

/* Line of text found, select color and print it. */

                tmp = tok.nextToken();
                if (tmp.equals("BLACK")) {
                    gc.setColor(Color.black);
                }
                if (tmp.equals("RED")) {
                    gc.setColor(Color.red);
                }
                if (tmp.equals("GREEN")) {
                    gc.setColor(Color.green);
                }
                if (tmp.equals("BLUE")) {
                    gc.setColor(Color.blue);
                }
                if (tmp.equals("YELLOW")) {
                    gc.setColor(Color.yellow);
                }
                gc.drawString(tok.nextToken().replace('|',' '),
                             Integer.parseInt(tok.nextToken()),
                             Integer.parseInt(tok.nextToken()));
            } else if (drawcommand.equals("LINE")) {

/* Line command found, calc the RC and draw the line with the current brush. */

                selectedbrush = (Image) wbu.brushes.get(new Integer(who));

                if (selectedbrush == null) {
                   wbu.brushes.put(new Integer(who), wbu.bs[0]);
                }
                selectedbrush = (Image) wbu.brushes.get(new Integer(who));

                px1 = Integer.parseInt(tok.nextToken()) -
                                       (selectedbrush.getWidth(this) / 2);
                py1 = Integer.parseInt(tok.nextToken()) -
                                       (selectedbrush.getHeight(this) / 2);
                px2 = Integer.parseInt(tok.nextToken()) -
                                       (selectedbrush.getWidth(this) / 2);
                py2 = Integer.parseInt(tok.nextToken()) -
                                       (selectedbrush.getHeight(this) / 2);
                if (Math.abs(px1 - px2) > Math.abs(py1 - py2)) {
                    k = py1;
                    l = (py2 - py1) / (float) (Math.abs(px2 - px1));
                    j = px1;
                    if (px1 < px2) {
                        i = 1;
                    } else {
                        i = -1;
                    }
                    while (j != px2) {
                        gc.drawImage(selectedbrush, j, (int) k, this);
                        k += l;
                        j += i;
                    }
                } else {
                    k = px1;
                    l = (px2 - px1) / (float) (Math.abs(py2-py1));
                    j = py1;
                    if (py1 < py2) {
                        i = 1;
                    } else {
                        i = -1;
                    }
                    while (j != py2) {
                        gc.drawImage(selectedbrush, (int) k, j, this);
                        k += l;
                        j += i;
                    }
                }
            } else if (drawcommand.equals("CIRCLE")) {

/*  Circle command found, calculate the radius and draw the circle with the
 *  current brush.
 */

                selectedbrush = (Image) wbu.brushes.get(new Integer(who));

                if (selectedbrush == null) {
                   wbu.brushes.put(new Integer(who), wbu.bs[0]);
                }
                selectedbrush = (Image) wbu.brushes.get(new Integer(who));

                px1 = Integer.parseInt(tok.nextToken()) -
                                       (selectedbrush.getWidth(this) / 2);
                py1 = Integer.parseInt(tok.nextToken()) -
                                       (selectedbrush.getHeight(this) / 2);
                px2 = Integer.parseInt(tok.nextToken()) -
                                       (selectedbrush.getWidth(this) / 2);
                py2 = Integer.parseInt(tok.nextToken()) -
                                       (selectedbrush.getHeight(this) / 2);

                k = Math.abs(px1 - px2);
                l = Math.abs(py1 - py2);
                radius = (int) Math.sqrt((k * k) + (l * l));
                drawCircle(selectedbrush, px1, py1, radius);
            } else if (drawcommand.equals("CLR")) {

/* Clear screen found, clear the drawing area. */

                gc.setColor(Color.white);
                gc.fillRect(0, 0, width, height);
            }
        }
    }    


    private void
    drawCirclePoints(Image brush, int cx, int cy, int dx, int dy) {
        gc.drawImage(brush, cx + dx, cy + dy, this);
        gc.drawImage(brush, cx - dx, cy + dy, this);
        gc.drawImage(brush, cx + dx, cy - dy, this);
        gc.drawImage(brush, cx - dx, cy - dy, this);
        gc.drawImage(brush, cx + dy, cy + dx, this);
        gc.drawImage(brush, cx - dy, cy + dx, this);
        gc.drawImage(brush, cx + dy, cy - dx, this);
        gc.drawImage(brush, cx - dy, cy - dx, this);
    }


    private void
    drawCircle(Image brush, int cx, int cy, int r) {
        int dx = 0;
        int dy = r;
        int p = 3 - 2*r;

        if (DrawingArea_Debug) {
            System.err.println("DrawingArea: drawCircle: cx: " + cx +
                                "  cy: " + cy + "  radius: " + r);
        }

        while (dx < dy) {
            drawCirclePoints(brush, cx, cy, dx, dy);
            if (p < 0) {
                p = p + 4*dx + 6;
            } else {
                p = p + 4*(dx - dy) + 10;
                dy--;
            }
            dx++;
        }
        if (dx == dy) {
            drawCirclePoints(brush, cx, cy, dx, dy);
        }
    }    


    public void
    paint(Graphics g) {
        if (DrawingArea_Debug) {
            System.err.println("DrawingArea: paint: image is " + image);
        }

        if (image != null) {
            g.drawImage(image, 0, 0, this);
        } else {
            update(g);
        }
    }    
}
