
/*
 * @(#)WhiteBoardServer.java	1.20 99/05/12
 *
 * Copyright (c) 1993-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package examples.whiteboard;

import com.sun.media.jsdt.*;

/**
 * This is just a simple server/client based proof-of-concept
 * implementation to make sure that the Shared Data classes have
 * all the required API in them.
 *
 * It's based on WhiteBoard; a test applet for the game server from
 * daniel@vpro.nl. It's been rewritten to use the Java Shared Data API,
 * and to make it easier to modify and maintain.
 *
 * @version     1.5 - 12th May 1999
 * @author      daniel@vpro.nl
 * @author      Rich Burridge
 */

public class
WhiteBoardServer implements WhiteBoardDebugFlags {

    private static String sessionName = "WBSession";


    public static void
    main(String args[]) {
        WhiteBoardClient client      = null;
	Session          wbSession   = null;
        URLString        url         = null;
        String           sessionType = null;
	String           hostname    = null;
        int              hostport    = 0;

        if (WhiteBoardServer_Debug) {
            System.err.println("WhiteBoardServer: main.");
        }

        hostname    = getHost(args);
        hostport    = getPort(args);
        sessionType = getType(args);
	url         = URLString.createSessionURL(hostname, hostport,
                                                 sessionType, sessionName);

        try {

/* Registry running?  Start it if it isn't. */

            if (RegistryFactory.registryExists(sessionType) == false) {
                RegistryFactory.startRegistry(sessionType);
            }

/* Create a session, [re]bind it and create a channel. */

            client = new WhiteBoardClient("Server");
	    wbSession = SessionFactory.createSession(client, url, true);
            wbSession.createChannel(client, "WBChannel", true, true, false);
            System.err.println("Setup and bound WhiteBoard server.");
        } catch (Exception e) {
            System.err.print("WhiteBoardServer: main: ");
	    System.err.println("shared data exception: " + e);
            e.printStackTrace();
        }
    }


    private static String
    getHost(String args[]) {
        String defHost = "stard";  /* Default host name for connections. */
        int length = args.length;

        if (WhiteBoardServer_Debug) {
            System.err.println("WhiteBoardServer: getHost.");
        }

        for (int i = 0; i < length; i++) {
            if (args[i].equals("-server")) {
                if (++i < length) {
                    return(args[i]);
                }
            }    
        }
        return(defHost);
    }


    private static int
    getPort(String args[]) {
	int defPort = 4462;   /* Default port number for connections. */
        int length = args.length;

	if (WhiteBoardServer_Debug) {
            System.err.println("WhiteBoardServer: getPort.");
        }

	for (int i = 0; i < length; i++) {
	    if (args[i].equals("-port")) {
		if (++i < length) {
                    return(Integer.parseInt(args[i]));
		}
	    }
	}
	return(defPort);
    }


    private static String
    getType(String args[]) {
        String defType = "socket";   /* Default Session type. */
        int length = args.length;

        if (WhiteBoardServer_Debug) {
            System.err.println("WhiteBoardServer: getType.");
        }
 
        for (int i = 0; i < length; i++) {
            if (args[i].equals("-type")) {
                if (++i < length) {
                    return(args[i]);
                }
            }
        }
        return(defType);
    }
}
